@@ -4,7 +4,7 @@ import shutil
 import tarfile
 import tempfile
 from pathlib import Path
-from typing import TYPE_CHECKING, Optional, Tuple
+from typing import TYPE_CHECKING, Dict, Optional, Tuple
 from urllib.parse import urlparse
 from zipfile import ZipFile, is_zipfile
 
@@ -12,7 +12,12 @@ from .cache_file import CacheFile
 from .common import PathOrStr, get_cache_dir
 from .file_lock import FileLock
 from .meta import Meta
-from .schemes import get_scheme_client, get_supported_schemes, hf_get_from_cache
+from .schemes import (
+    SchemeClient,
+    get_scheme_client,
+    get_supported_schemes,
+    hf_get_from_cache,
+)
 from .util import (
     _lock_file_path,
     _meta_file_path,
@@ -27,6 +32,31 @@ if TYPE_CHECKING:
 logger = logging.getLogger("cached_path")
 
 
+def _is_rarfile(path: PathOrStr, url_or_filename: PathOrStr) -> bool:
+    try:
+        import rarfile
+
+        return rarfile.is_rarfile(path)
+    except ImportError:
+        if str(url_or_filename).lower().endswith(".rar"):
+            import warnings
+
+            warnings.warn(
+                f"The file has a 'rar' extension but rarfile is unavailable: {url_or_filename}"
+                "\nSee https://rarfile.readthedocs.io/"
+            )
+
+        return False
+
+
+def _is_archive(file_path: PathOrStr, url_or_filename) -> bool:
+    return (
+        is_zipfile(file_path)
+        or tarfile.is_tarfile(file_path)
+        or _is_rarfile(file_path, url_or_filename)
+    )
+
+
 def cached_path(
     url_or_filename: PathOrStr,
     cache_dir: Optional[PathOrStr] = None,
@@ -34,6 +64,7 @@ def cached_path(
     force_extract: bool = False,
     quiet: bool = False,
     progress: Optional["Progress"] = None,
+    headers: Optional[Dict[str, str]] = None,
 ) -> Path:
     """
     Given something that might be a URL or local path, determine which.
@@ -113,6 +144,11 @@ def cached_path(
         A custom progress display to use. If not set and ``quiet=False``, a default display
         from :func:`~cached_path.get_download_progress()` will be used.
 
+    headers :
+        Custom headers to add to HTTP requests.
+        Example: ``{"Authorization": "Bearer YOUR_TOKEN"}`` for private resources.
+        Only used for HTTP/HTTPS resources.
+
     Returns
     -------
     :class:`pathlib.Path`
@@ -153,6 +189,7 @@ def cached_path(
             force_extract=force_extract,
             quiet=quiet,
             progress=progress,
+            headers=headers,
         )
         if not cached_archive_path.is_dir():
             raise ValueError(
@@ -171,13 +208,16 @@ def cached_path(
 
     if parsed.scheme in get_supported_schemes():
         # URL, so get it from the cache (downloading if necessary)
-        file_path, etag = get_from_cache(url_or_filename, cache_dir, quiet=quiet, progress=progress)
+        file_path, etag = get_from_cache(
+            url_or_filename, cache_dir, quiet=quiet, progress=progress, headers=headers
+        )
 
-        if extract_archive and (is_zipfile(file_path) or tarfile.is_tarfile(file_path)):
+        if extract_archive and _is_archive(file_path, url_or_filename):
             # This is the path the file should be extracted to.
             # For example ~/.cached_path/cache/234234.21341 -> ~/.cached_path/cache/234234.21341-extracted
             extraction_path = file_path.parent / (file_path.name + "-extracted")
-
+    elif parsed.scheme == "file":
+        return cached_path(url_or_filename.replace("file://", "", 1))
     else:
         orig_url_or_filename = url_or_filename
         url_or_filename = Path(url_or_filename).expanduser()
@@ -189,12 +229,7 @@ def cached_path(
             file_path = url_or_filename
             # Normalize the path.
             url_or_filename = url_or_filename.resolve()
-
-            if (
-                extract_archive
-                and file_path.is_file()
-                and (is_zipfile(file_path) or tarfile.is_tarfile(file_path))
-            ):
+            if extract_archive and file_path.is_file() and _is_archive(file_path, url_or_filename):
                 # We'll use a unique directory within the cache to root to extract the archive to.
                 # The name of the directory is a hash of the resource file path and it's modification
                 # time. That way, if the file changes, we'll know when to extract it again.
@@ -203,11 +238,9 @@ def cached_path(
                     + "-extracted"
                 )
                 extraction_path = cache_dir / extraction_name
-
         elif parsed.scheme == "":
             # File, but it doesn't exist.
             raise FileNotFoundError(f"file {url_or_filename} not found")
-
         else:
             # Something unknown
             raise ValueError(f"unable to parse {orig_url_or_filename} as a URL or as a local path")
@@ -239,15 +272,18 @@ def cached_path(
             # during the extraction process so we don't end up with a corrupted cache.
             tmp_extraction_dir = tempfile.mkdtemp(dir=os.path.split(extraction_path)[0])
             try:
-                if is_zipfile(file_path):
-                    with ZipFile(file_path, "r") as zip_file:
-                        zip_file.extractall(tmp_extraction_dir)
-                        zip_file.close()
+                if tarfile.is_tarfile(file_path):
+                    with tarfile.open(file_path) as tar_file:
+                        check_tarfile(tar_file)
+                        tar_file.extractall(tmp_extraction_dir)
+                elif _is_rarfile(file_path, url_or_filename):
+                    import rarfile
+
+                    with rarfile.RarFile(file_path) as rar_file:
+                        rar_file.extractall(tmp_extraction_dir)
                 else:
-                    tar_file = tarfile.open(file_path)
-                    check_tarfile(tar_file)
-                    tar_file.extractall(tmp_extraction_dir)
-                    tar_file.close()
+                    with ZipFile(file_path) as zip_file:
+                        zip_file.extractall(tmp_extraction_dir)
                 # Extraction was successful, rename temp directory to final
                 # cache directory and dump the meta data.
                 os.replace(tmp_extraction_dir, extraction_path)
@@ -271,6 +307,9 @@ def get_from_cache(
     cache_dir: Optional[PathOrStr] = None,
     quiet: bool = False,
     progress: Optional["Progress"] = None,
+    no_downloads: bool = False,
+    _client: Optional[SchemeClient] = None,
+    headers: Optional[Dict[str, str]] = None,
 ) -> Tuple[Path, Optional[str]]:
     """
     Given a URL, look for the corresponding dataset in the local cache.
@@ -281,8 +320,7 @@ def get_from_cache(
 
     cache_dir = Path(cache_dir if cache_dir else get_cache_dir()).expanduser()
     cache_dir.mkdir(parents=True, exist_ok=True)
-
-    client = get_scheme_client(url)
+    client = _client or get_scheme_client(url, headers=headers)
 
     # Get eTag to add to filename, if it exists.
     try:
@@ -300,8 +338,7 @@ def get_from_cache(
         latest_cached = find_latest_cached(url, cache_dir)
         if latest_cached:
             logger.info(
-                "ETag request failed with recoverable error, using latest cached "
-                "version of %s: %s",
+                "ETag request failed with recoverable error, using latest cached version of %s: %s",
                 url,
                 latest_cached,
             )
@@ -309,8 +346,7 @@ def get_from_cache(
             return latest_cached, meta.etag
         else:
             logger.error(
-                "ETag request failed with recoverable error, "
-                "but no cached version of %s could be found",
+                "ETag request failed with recoverable error, but no cached version of %s could be found",
                 url,
             )
             raise
@@ -329,6 +365,8 @@ def get_from_cache(
     with FileLock(_lock_file_path(cache_path), read_only_ok=True):
         if os.path.exists(cache_path):
             logger.info("cache of %s is up-to-date", url)
+        elif no_downloads:
+            raise FileNotFoundError(cache_path)
         else:
             size = client.get_size()
             with CacheFile(cache_path) as cache_file:
@@ -343,7 +381,7 @@ def get_from_cache(
                     progress.start()
 
                 try:
-                    display_url = url if len(url) <= 30 else f"\N{horizontal ellipsis}{url[-30:]}"
+                    display_url = url if len(url) <= 30 else f"\N{HORIZONTAL ELLIPSIS}{url[-30:]}"
                     task_id = progress.add_task(f"Downloading [cyan i]{display_url}[/]", total=size)
                     writer_with_progress = BufferedWriterWithProgress(cache_file, progress, task_id)
                     client.get_resource(writer_with_progress)
