@@ -4,7 +4,7 @@ import time
 from dataclasses import asdict, dataclass
 from typing import Optional, Set
 
-from cached_path.common import PathOrStr
+from .common import PathOrStr
 
 
 @dataclass
@@ -57,7 +57,7 @@ class Meta:
         cached_path: PathOrStr,
         *,
         etag: Optional[str] = None,
-        extraction_dir: bool = False
+        extraction_dir: bool = False,
     ) -> "Meta":
         return cls(  # type: ignore
             resource=str(resource),
@@ -99,7 +99,7 @@ class Meta:
             return os.path.getsize(path)
         inodes: Set[int] = set()
         total_size = 0
-        for dirpath, dirnames, filenames in os.walk(path):
+        for dirpath, _, filenames in os.walk(str(path)):
             for f in filenames:
                 fp = os.path.join(dirpath, f)
                 # skip if it is symbolic link or the same as a file we've already accounted
