@@ -3,11 +3,13 @@ Google Cloud Storage.
 """
 
 import io
+from functools import cache
 from typing import Optional, Tuple
 
 from google.api_core.exceptions import NotFound
 from google.auth.exceptions import DefaultCredentialsError
 from google.cloud import storage
+from google.cloud.storage.blob import Blob
 from google.cloud.storage.retry import DEFAULT_RETRY
 
 from ..common import _split_cloud_path
@@ -27,8 +29,8 @@ class GsClient(SchemeClient):
             try:
                 self.blob.reload()
                 self._loaded = True
-            except NotFound:
-                raise FileNotFoundError(self.resource)
+            except NotFound as exc:
+                raise FileNotFoundError(self.resource) from exc
 
     def get_etag(self) -> Optional[str]:
         self.load()
@@ -42,17 +44,28 @@ class GsClient(SchemeClient):
         self.load()
         self.blob.download_to_file(temp_file, checksum="md5", retry=DEFAULT_RETRY)
 
+    def get_bytes_range(self, index: int, length: int) -> bytes:
+        self.load()
+        return self.blob.download_as_bytes(start=index, end=index + length - 1)
+
     @staticmethod
     def split_gcs_path(resource: str) -> Tuple[str, str]:
         return _split_cloud_path(resource, "gs")
 
     @staticmethod
-    def get_gcs_blob(resource: str) -> storage.blob.Blob:
-        try:
-            gcs_resource = storage.Client()
-        except DefaultCredentialsError:
-            gcs_resource = storage.Client.create_anonymous_client()
+    def get_gcs_blob(resource: str) -> Blob:
+        gcs_client = _get_gcs_client()
         bucket_name, gcs_path = GsClient.split_gcs_path(resource)
-        bucket = gcs_resource.bucket(bucket_name)
+        bucket = gcs_client.bucket(bucket_name)
         blob = bucket.blob(gcs_path)
         return blob
+
+
+@cache
+def _get_gcs_client() -> storage.Client:
+    try:
+        client = storage.Client()
+    except DefaultCredentialsError:
+        client = storage.Client.create_anonymous_client()
+
+    return client
